@@ -2,12 +2,14 @@
 import json
 import time
 import copy
+import dpath.util
 
 import singer
 from singer import utils, metadata
 from tap_kafka.local_store import LocalStore
 from kafka import KafkaConsumer, OffsetAndMetadata, TopicPartition
-from jsonpath_ng import parse
+
+from .errors import InvalidBookmarkException
 
 LOGGER = singer.get_logger('tap_kafka')
 
@@ -49,13 +51,19 @@ def send_schema_message(stream):
 
 def update_bookmark(state, topic, timestamp):
     """Update bookmark with a new timestamp"""
-    return singer.write_bookmark(state, topic, 'timestamp', timestamp or 0)
+    try:
+        timestamp = float(timestamp) if timestamp else 0
+    except ValueError:
+        raise InvalidBookmarkException(f'The timestamp in the bookmark for {topic} stream is not numeric')
+
+    return singer.write_bookmark(state, topic, 'timestamp', timestamp)
 
 
 def init_local_store(kafka_config):
     LOGGER.info('Initialising local store at %s', kafka_config['local_store_dir'])
     return LocalStore(
         directory=kafka_config['local_store_dir'],
+        batch_size_rows=kafka_config['local_store_batch_size_rows'],
         topic=kafka_config['topic'])
 
 
@@ -93,9 +101,12 @@ def kafka_message_to_singer_record(message, topic, primary_keys):
     # Add primary keys to the record message
     for key in primary_keys:
         pk_selector = primary_keys[key]
-        match = parse(pk_selector).find(message.value)
-        if match:
-            record[key] = match[0].value
+        try:
+            record[key] = dpath.util.get(message.value, pk_selector)
+        # Do not fail if PK not found in the message.
+        # Continue without adding the extracted PK to the message
+        except KeyError:
+            pass
 
     return record
 
@@ -103,9 +114,10 @@ def kafka_message_to_singer_record(message, topic, primary_keys):
 def consume_kafka_message(message, topic, primary_keys, local_store):
     """Insert single kafka message into the internal store"""
     singer_record = kafka_message_to_singer_record(message, topic, primary_keys)
-    insert_ts = local_store.insert(singer.format_message(singer.RecordMessage(stream=topic,
-                                                                              record=singer_record,
-                                                                              time_extracted=utils.now())))
+    singer_record_message = singer.format_message(singer.RecordMessage(stream=topic,
+                                                                       record=singer_record,
+                                                                       time_extracted=utils.now()))
+    insert_ts = local_store.insert(singer_record_message)
     return insert_ts
 
 
@@ -124,11 +136,13 @@ def read_kafka_topic(consumer, local_store, kafka_config, state, fn_get_args):
     topic = kafka_config['topic']
     primary_keys = kafka_config['primary_keys']
     max_runtime_ms = kafka_config['max_runtime_ms']
+    commit_interval_ms = kafka_config['commit_interval_ms']
     batch_size_rows = kafka_config['batch_size_rows']
     local_store_cleanup_ts = time.time()
     received_messages = 0
     last_consumed_ts = 0
     start_time = 0
+    last_commit_time = 0
 
     # Send singer ACTIVATE message
     send_activate_version_message(state, topic)
@@ -144,11 +158,19 @@ def read_kafka_topic(consumer, local_store, kafka_config, state, fn_get_args):
         if not start_time:
             start_time = time.time()
 
+        # Initialise the last_commit_time after the first message
+        if not last_commit_time:
+            last_commit_time = time.time()
+
         # Generate and insert singer message into local store
         last_consumed_ts = consume_kafka_message(message, topic, primary_keys, local_store)
 
-        # Commit offsets because we processed the message
-        commit_kafka_consumer(consumer, message.topic, message.partition, message.offset)
+        # Commit periodically
+        if last_consumed_ts - last_commit_time > commit_interval_ms / 1000:
+            # Persist everything in the local store to disk and send commit message to kafka
+            local_store.persist_messages()
+            commit_kafka_consumer(consumer, message.topic, message.partition, message.offset)
+            last_commit_time = time.time()
 
         # Log message stats periodically
         received_messages += 1
@@ -158,7 +180,7 @@ def read_kafka_topic(consumer, local_store, kafka_config, state, fn_get_args):
 
         # Every UPDATE_BOOKMARK_PERIOD, update the bookmark and send state message
         if received_messages % UPDATE_BOOKMARK_PERIOD == 0:
-            state = update_bookmark(state, topic, last_consumed_ts)
+            state = update_bookmark(state, topic, local_store.last_persisted_ts)
             LOGGER.debug("Updating bookmark and inserting to local store: %s", state)
             local_store.insert(singer.format_message(singer.StateMessage(value=copy.deepcopy(state))))
 
@@ -166,7 +188,7 @@ def read_kafka_topic(consumer, local_store, kafka_config, state, fn_get_args):
         if received_messages % batch_size_rows == 0:
             LOGGER.debug('Sending %d unprocessed messages from local store...', batch_size_rows)
             local_store.flush_after(last_flush_ts)
-            last_flush_ts = last_consumed_ts
+            last_flush_ts = local_store.last_persisted_ts
 
         now = time.time()
         # Every CLEANUP_LOCAL_STORE_INTERVAL delete the processed items from the local store
@@ -192,6 +214,8 @@ def read_kafka_topic(consumer, local_store, kafka_config, state, fn_get_args):
     if last_consumed_ts:
         state = update_bookmark(state, topic, last_consumed_ts)
         local_store.insert(singer.format_message(singer.StateMessage(value=copy.deepcopy(state))))
+        local_store.persist_messages()
+        commit_kafka_consumer(consumer, message.topic, message.partition, message.offset)
 
     return last_flush_ts
 
